'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ParseAnalyticsFacade = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _CoreManager = require('./CoreManager');

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

/**
 * Parse.Analytics provides an interface to Parse's logging and analytics
 * backend.
 *
 * @class Parse.Analytics
 * @static
 * @hideconstructor
 */

/**
 * Tracks the occurrence of a custom event with additional dimensions.
 * Parse will store a data point at the time of invocation with the given
 * event name.
 *
 * Dimensions will allow segmentation of the occurrences of this custom
 * event. Keys and values should be {@code String}s, and will throw
 * otherwise.
 *
 * To track a user signup along with additional metadata, consider the
 * following:
 * <pre>
 * var dimensions = {
 *  gender: 'm',
 *  source: 'web',
 *  dayType: 'weekend'
 * };
 * Parse.Analytics.track('signup', dimensions);
 * </pre>
 *
 * There is a default limit of 8 dimensions per event tracked.
 *
 * @method track
 * @name Parse.Analytics.track
 * @param {String} name The name of the custom event to report to Parse as
 * having happened.
 * @param {Object} dimensions The dictionary of information by which to
 * segment this event.
 * @param {Object} options A Backbone-style callback object.
 * @return {Parse.Promise} A promise that is resolved when the round-trip
 * to the server completes.
 */

var ParseAnalyticsFacade = exports.ParseAnalyticsFacade = function () {
  function ParseAnalyticsFacade(coreManager) {
    (0, _classCallCheck3.default)(this, ParseAnalyticsFacade);

    this._CoreManager = coreManager;
    this._CoreManager.setAnalyticsFacade(this);
    this._CoreManager.setAnalyticsController(new DefaultController(this._CoreManager));
  }

  (0, _createClass3.default)(ParseAnalyticsFacade, [{
    key: 'track',
    value: function (name, dimensions, options) {
      name = name || '';
      name = name.replace(/^\s*/, '');
      name = name.replace(/\s*$/, '');
      if (name.length === 0) {
        throw new TypeError('A name for the custom event must be provided');
      }

      for (var key in dimensions) {
        if (typeof key !== 'string' || typeof dimensions[key] !== 'string') {
          throw new TypeError('track() dimensions expects keys and values of type "string".');
        }
      }

      options = options || {};
      return this._CoreManager.getAnalyticsController().track(name, dimensions)._thenRunCallbacks(options);
    }
  }]);
  return ParseAnalyticsFacade;
}(); /**
      * Copyright (c) 2015-present, Parse, LLC.
      * All rights reserved.
      *
      * This source code is licensed under the BSD-style license found in the
      * LICENSE file in the root directory of this source tree. An additional grant
      * of patent rights can be found in the PATENTS file in the same directory.
      *
      * 
      */

var DefaultController = function () {
  function DefaultController(coreManager) {
    (0, _classCallCheck3.default)(this, DefaultController);

    this._CoreManager = coreManager;
  }

  (0, _createClass3.default)(DefaultController, [{
    key: 'track',
    value: function (name, dimensions) {
      var RESTController = this._CoreManager.getRESTController();
      return RESTController.request('POST', 'events/' + name, { dimensions: dimensions });
    }
  }]);
  return DefaultController;
}();

var _AnalyticsFacade = new ParseAnalyticsFacade(_CoreManager.CoreManager);