'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ParseHooks = exports.ParseHooksFacade = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _CoreManager = require('./CoreManager');

var _decode = require('./decode');

var _decode2 = _interopRequireDefault(_decode);

var _encode = require('./encode');

var _encode2 = _interopRequireDefault(_encode);

var _ParseError = require('./ParseError');

var _ParseError2 = _interopRequireDefault(_ParseError);

var _ParsePromise = require('./ParsePromise');

var _ParsePromise2 = _interopRequireDefault(_ParsePromise);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

var ParseHooksFacade = exports.ParseHooksFacade = function () {
  function ParseHooksFacade(coreManager) {
    (0, _classCallCheck3.default)(this, ParseHooksFacade);

    this._CoreManager = coreManager;
    this._CoreManager.setHooksFacade(this);
    this._CoreManager.setHooksController(new DefaultController(this._CoreManager));
  }

  (0, _createClass3.default)(ParseHooksFacade, [{
    key: 'getFunctions',
    value: function () {
      return this._CoreManager.getHooksController().get("functions");
    }
  }, {
    key: 'getTriggers',
    value: function () {
      return this._CoreManager.getHooksController().get("triggers");
    }
  }, {
    key: 'getFunction',
    value: function (name) {
      return this._CoreManager.getHooksController().get("functions", name);
    }
  }, {
    key: 'getTrigger',
    value: function (className, triggerName) {
      return this._CoreManager.getHooksController().get("triggers", className, triggerName);
    }
  }, {
    key: 'createFunction',
    value: function (functionName, url) {
      return this.create({ functionName: functionName, url: url });
    }
  }, {
    key: 'createTrigger',
    value: function (className, triggerName, url) {
      return this.create({ className: className, triggerName: triggerName, url: url });
    }
  }, {
    key: 'create',
    value: function (hook) {
      return this._CoreManager.getHooksController().create(hook);
    }
  }, {
    key: 'updateFunction',
    value: function (functionName, url) {
      return this.update({ functionName: functionName, url: url });
    }
  }, {
    key: 'updateTrigger',
    value: function (className, triggerName, url) {
      return this.update({ className: className, triggerName: triggerName, url: url });
    }
  }, {
    key: 'update',
    value: function (hook) {
      return this._CoreManager.getHooksController().update(hook);
    }
  }, {
    key: 'removeFunction',
    value: function (functionName) {
      return this.remove({ functionName: functionName });
    }
  }, {
    key: 'removeTrigger',
    value: function (className, triggerName) {
      return this.remove({ className: className, triggerName: triggerName });
    }
  }, {
    key: 'remove',
    value: function (hook) {
      return this._CoreManager.getHooksController().remove(hook);
    }
  }]);
  return ParseHooksFacade;
}();

var DefaultController = function () {
  function DefaultController(coreManager) {
    (0, _classCallCheck3.default)(this, DefaultController);

    this._CoreManager = coreManager;
  }

  (0, _createClass3.default)(DefaultController, [{
    key: 'get',
    value: function (type, functionName, triggerName) {
      var url = "/hooks/" + type;
      if (functionName) {
        url += "/" + functionName;
        if (triggerName) {
          url += "/" + triggerName;
        }
      }
      return this.sendRequest("GET", url);
    }
  }, {
    key: 'create',
    value: function (hook) {
      var url;
      if (hook.functionName && hook.url) {
        url = "/hooks/functions";
      } else if (hook.className && hook.triggerName && hook.url) {
        url = "/hooks/triggers";
      } else {
        return _promise2.default.reject({ error: 'invalid hook declaration', code: 143 });
      }
      return this.sendRequest("POST", url, hook);
    }
  }, {
    key: 'remove',
    value: function (hook) {
      var url;
      if (hook.functionName) {
        url = "/hooks/functions/" + hook.functionName;
        delete hook.functionName;
      } else if (hook.className && hook.triggerName) {
        url = "/hooks/triggers/" + hook.className + "/" + hook.triggerName;
        delete hook.className;
        delete hook.triggerName;
      } else {
        return _promise2.default.reject({ error: 'invalid hook declaration', code: 143 });
      }
      return this.sendRequest("PUT", url, { "__op": "Delete" });
    }
  }, {
    key: 'update',
    value: function (hook) {
      var url;
      if (hook.functionName && hook.url) {
        url = "/hooks/functions/" + hook.functionName;
        delete hook.functionName;
      } else if (hook.className && hook.triggerName && hook.url) {
        url = "/hooks/triggers/" + hook.className + "/" + hook.triggerName;
        delete hook.className;
        delete hook.triggerName;
      } else {
        return _promise2.default.reject({ error: 'invalid hook declaration', code: 143 });
      }
      return this.sendRequest('PUT', url, hook);
    }
  }, {
    key: 'sendRequest',
    value: function (method, url, body) {
      var _this = this;

      return this._CoreManager.getRESTController().request(method, url, body, { useMasterKey: true }).then(function (res) {
        var decoded = (0, _decode2.default)(res, _this._CoreManager);
        if (decoded) {
          return _ParsePromise2.default.as(decoded);
        }
        return _ParsePromise2.default.error(new _ParseError2.default(_ParseError2.default.INVALID_JSON, 'The server returned an invalid response.'));
      });
    }
  }]);
  return DefaultController;
}();

var ParseHooks = exports.ParseHooks = new ParseHooksFacade(_CoreManager.CoreManager);

exports.default = ParseHooks;