'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ParseTasks = exports.ParseTasksFacade = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _CoreManager = require('./CoreManager');

var _decode = require('./decode');

var _decode2 = _interopRequireDefault(_decode);

var _encode = require('./encode');

var _encode2 = _interopRequireDefault(_encode);

var _ParseError = require('./ParseError');

var _ParseError2 = _interopRequireDefault(_ParseError);

var _ParsePromise = require('./ParsePromise');

var _ParsePromise2 = _interopRequireDefault(_ParsePromise);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

var ParseTasksFacade = exports.ParseTasksFacade = function () {
  function ParseTasksFacade(coreManager) {
    (0, _classCallCheck3.default)(this, ParseTasksFacade);

    this._CoreManager = coreManager;
    this._CoreManager.setTasksFacade(this);
    this._CoreManager.setTasksController(new DefaultController(this._CoreManager));
  }

  (0, _createClass3.default)(ParseTasksFacade, [{
    key: 'getFunctions',
    value: function () {
      return this._CoreManager.getTasksController().get("functions");
    }
  }, {
    key: 'getTriggers',
    value: function () {
      return this._CoreManager.getTasksController().get("triggers");
    }
  }, {
    key: 'getFunction',
    value: function (name) {
      return this._CoreManager.getTasksController().get("functions", name);
    }
  }, {
    key: 'getTrigger',
    value: function (className, triggerName) {
      return this._CoreManager.getTasksController().get("triggers", className, triggerName);
    }
  }, {
    key: 'createFunction',
    value: function (functionName, taskDefinition, taskRepeat) {
      return this.create({ functionName: functionName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
    }
  }, {
    key: 'createTrigger',
    value: function (className, triggerName, taskDefinition, taskRepeat) {
      return this.create({ className: className, triggerName: triggerName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
    }
  }, {
    key: 'create',
    value: function (task) {
      return this._CoreManager.getTasksController().create(task);
    }
  }, {
    key: 'updateFunction',
    value: function (functionName, taskDefinition, taskRepeat) {
      return this.update({ functionName: functionName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
    }
  }, {
    key: 'updateTrigger',
    value: function (className, triggerName, taskDefinition, taskRepeat) {
      return this.update({ className: className, triggerName: triggerName, taskDefinition: taskDefinition, taskRepeat: taskRepeat });
    }
  }, {
    key: 'update',
    value: function (task) {
      return this._CoreManager.getTasksController().update(task);
    }
  }, {
    key: 'removeFunction',
    value: function (functionName) {
      return this.remove({ functionName: functionName });
    }
  }, {
    key: 'removeTrigger',
    value: function (className, triggerName) {
      return this.remove({ className: className, triggerName: triggerName });
    }
  }, {
    key: 'remove',
    value: function (task) {
      return this._CoreManager.getTasksController().remove(task);
    }
  }]);
  return ParseTasksFacade;
}();

var DefaultController = function () {
  function DefaultController(coreManager) {
    (0, _classCallCheck3.default)(this, DefaultController);

    this._CoreManager = coreManager;
  }

  (0, _createClass3.default)(DefaultController, [{
    key: 'get',
    value: function (type, functionName, triggerName) {
      var url = "/tasks/" + type;
      if (functionName) {
        url += "/" + functionName;
        if (triggerName) {
          url += "/" + triggerName;
        }
      }
      return this.sendRequest("GET", url);
    }
  }, {
    key: 'create',
    value: function (task) {
      var url;
      if (task.functionName && task.taskDefinition) {
        url = "/tasks/functions";
      } else if (task.className && task.triggerName && task.taskDefinition) {
        url = "/tasks/triggers";
      } else {
        return _promise2.default.reject({ error: 'invalid task declaration', code: 143 });
      }
      return this.sendRequest("POST", url, task);
    }
  }, {
    key: 'remove',
    value: function (task) {
      var url;
      if (task.functionName) {
        url = "/tasks/functions/" + task.functionName;
        delete task.functionName;
      } else if (task.className && task.triggerName) {
        url = "/tasks/triggers/" + task.className + "/" + task.triggerName;
        delete task.className;
        delete task.triggerName;
      } else {
        return _promise2.default.reject({ error: 'invalid task declaration', code: 143 });
      }
      return this.sendRequest("PUT", url, { "__op": "Delete" });
    }
  }, {
    key: 'update',
    value: function (task) {
      var url;
      if (task.functionName && task.taskDefinition) {
        url = "/tasks/functions/" + task.functionName;
        delete task.functionName;
      } else if (task.className && task.triggerName && task.taskDefinition) {
        url = "/tasks/triggers/" + task.className + "/" + task.triggerName;
        delete task.className;
        delete task.triggerName;
      } else {
        return _promise2.default.reject({ error: 'invalid task declaration', code: 143 });
      }
      return this.sendRequest('PUT', url, task);
    }
  }, {
    key: 'sendRequest',
    value: function (method, url, body) {
      var _this = this;

      return this._CoreManager.getRESTController().request(method, url, body, { useMasterKey: true }).then(function (res) {
        var decoded = (0, _decode2.default)(res, _this._CoreManager);
        if (decoded) {
          return _ParsePromise2.default.as(decoded);
        }
        return _ParsePromise2.default.error(new _ParseError2.default(_ParseError2.default.INVALID_JSON, 'The server returned an invalid response.'));
      });
    }
  }]);
  return DefaultController;
}();

var ParseTasks = exports.ParseTasks = new ParseTasksFacade(_CoreManager.CoreManager);

exports.default = ParseTasks;