'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _ObjectStateMutations = require('./ObjectStateMutations');

var ObjectStateMutations = _interopRequireWildcard(_ObjectStateMutations);

function _interopRequireWildcard(obj) {
  if (obj && obj.__esModule) {
    return obj;
  } else {
    var newObj = {};if (obj != null) {
      for (var key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
      }
    }newObj.default = obj;return newObj;
  }
}

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

var SingleInstanceStateController = function () {
  function SingleInstanceStateController() {
    (0, _classCallCheck3.default)(this, SingleInstanceStateController);

    this.objectState = {};
  }

  (0, _createClass3.default)(SingleInstanceStateController, [{
    key: 'getState',
    value: function (obj) {
      var classData = this.objectState[obj.className];
      if (classData) {
        return classData[obj.id] || null;
      }
      return null;
    }
  }, {
    key: 'initializeState',
    value: function (obj, initial) {
      var state = this.getState(obj);
      if (state) {
        return state;
      }
      if (!this.objectState[obj.className]) {
        this.objectState[obj.className] = {};
      }
      if (!initial) {
        initial = ObjectStateMutations.defaultState();
      }
      state = this.objectState[obj.className][obj.id] = initial;
      return state;
    }
  }, {
    key: 'removeState',
    value: function (obj) {
      var state = this.getState(obj);
      if (state === null) {
        return null;
      }
      delete this.objectState[obj.className][obj.id];
      return state;
    }
  }, {
    key: 'getServerData',
    value: function (obj) {
      var state = this.getState(obj);
      if (state) {
        return state.serverData;
      }
      return {};
    }
  }, {
    key: 'setServerData',
    value: function (obj, attributes) {
      var serverData = this.initializeState(obj).serverData;
      ObjectStateMutations.setServerData(serverData, attributes);
    }
  }, {
    key: 'getPendingOps',
    value: function (obj) {
      var state = this.getState(obj);
      if (state) {
        return state.pendingOps;
      }
      return [{}];
    }
  }, {
    key: 'setPendingOp',
    value: function (obj, attr, op) {
      var pendingOps = this.initializeState(obj).pendingOps;
      ObjectStateMutations.setPendingOp(pendingOps, attr, op);
    }
  }, {
    key: 'pushPendingState',
    value: function (obj) {
      var pendingOps = this.initializeState(obj).pendingOps;
      ObjectStateMutations.pushPendingState(pendingOps);
    }
  }, {
    key: 'popPendingState',
    value: function (obj) {
      var pendingOps = this.initializeState(obj).pendingOps;
      return ObjectStateMutations.popPendingState(pendingOps);
    }
  }, {
    key: 'mergeFirstPendingState',
    value: function (obj) {
      var pendingOps = this.getPendingOps(obj);
      ObjectStateMutations.mergeFirstPendingState(pendingOps);
    }
  }, {
    key: 'getObjectCache',
    value: function (obj) {
      var state = this.getState(obj);
      if (state) {
        return state.objectCache;
      }
      return {};
    }
  }, {
    key: 'estimateAttribute',
    value: function (obj, attr) {
      var serverData = this.getServerData(obj);
      var pendingOps = this.getPendingOps(obj);
      return ObjectStateMutations.estimateAttribute(serverData, pendingOps, obj.className, obj.id, obj.coreManager, attr);
    }
  }, {
    key: 'estimateAttributes',
    value: function (obj) {
      var serverData = this.getServerData(obj);
      var pendingOps = this.getPendingOps(obj);
      return ObjectStateMutations.estimateAttributes(serverData, pendingOps, obj.className, obj.id, obj.coreManager);
    }
  }, {
    key: 'commitServerChanges',
    value: function (obj, changes) {
      var state = this.initializeState(obj);
      ObjectStateMutations.commitServerChanges(state.serverData, state.objectCache, changes);
    }
  }, {
    key: 'enqueueTask',
    value: function (obj, task) {
      var state = this.initializeState(obj);
      return state.tasks.enqueue(task);
    }
  }, {
    key: 'clearAllState',
    value: function () {
      this.objectState = {};
    }
  }, {
    key: 'duplicateState',
    value: function (source, dest) {
      dest.id = source.id;
    }
  }]);
  return SingleInstanceStateController;
}(); /**
      * Copyright (c) 2015-present, Parse, LLC.
      * All rights reserved.
      *
      * This source code is licensed under the BSD-style license found in the
      * LICENSE file in the root directory of this source tree. An additional grant
      * of patent rights can be found in the PATENTS file in the same directory.
      *
      * 
      */

exports.default = SingleInstanceStateController;