'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

exports.makeDatonis = makeDatonis;

var _CoreManager = require('./CoreManager');

var _ParsePromise = require('./ParsePromise');

var _ParsePromise2 = _interopRequireDefault(_ParsePromise);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

function makeDatonis(coreManager) {
  return function (_DatonisFacade) {
    (0, _inherits3.default)(_class, _DatonisFacade);

    function _class(datonisToken, version) {
      (0, _classCallCheck3.default)(this, _class);
      return (0, _possibleConstructorReturn3.default)(this, (_class.__proto__ || (0, _getPrototypeOf2.default)(_class)).call(this, datonisToken, version, coreManager));
    }

    return _class;
  }(DatonisFacade);
}

var DatonisFacade = function () {

  /**
   * @param {(String)} datonisToken Datonis auth token
   * @param {(Number)} version Datonis api version
   */

  /**
   * @property datonisToken
   * @type String
   */
  function DatonisFacade(datonisToken, version, coreManager) {
    (0, _classCallCheck3.default)(this, DatonisFacade);

    this._CoreManager = coreManager;

    this.XHR = null;
    if (typeof XMLHttpRequest !== 'undefined') {
      this.XHR = XMLHttpRequest;
    }

    this.XHR = require('xmlhttprequest').XMLHttpRequest;


    if (typeof datonisToken === 'string') {
      this.datonisToken = datonisToken;
    } else {
      throw new TypeError('Datonis token must be of type string.');
    }
    version = version || 3;
    if (typeof version === 'number') {
      this.version = version;
    } else {
      throw new TypeError('Datonis version must be of type number.');
    }
  }

  /**
   * Get things.
   * @param {Hash} data
   * @param {Hash} options
   * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
   */

  /**
   * @property version
   * @type Number
   */

  (0, _createClass3.default)(DatonisFacade, [{
    key: 'getThings',
    value: function () {
      var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var url = this.getDatonisUrl() + 'api/v' + this.version + '/things';

      return this.ajax('GET', url, data, options);
    }

    /**
     * Get things.
     * @param {Hash} data
     * @param {Hash} options
     * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
     */

  }, {
    key: 'getThingRawEvents',
    value: function () {
      var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var url = this.getDatonisUrl() + 'api/v' + this.version + '/datonis_query/thing_data';

      return this.ajax('POST', url, data, options);
    }

    /**
     * Get things.
     * @param {Hash} data
     * @param {Hash} options
     * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
     */

  }, {
    key: 'getThingWaypoints',
    value: function (thing_key) {
      var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      var url = this.getDatonisUrl() + 'api/v' + this.version + '/things/' + thing_key + '/waypoints';

      return this.ajax('GET', url, data, options);
    }

    /**
     * Get thing aggregated data. Function defination can be improved in future in case any additional computation required
     * from sdk.
     * @param {Hash} data
     * @param {Hash} options
     * @return {Parse.Promise} A promise that is fulfilled when the ajax completes.
     */

  }, {
    key: 'getThingAggregateData',
    value: function () {
      var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var url = this.getDatonisUrl() + 'api/v' + this.version + '/datonis_query/thing_aggregated_data';

      return this.ajax('POST', url, data, options);
    }
  }, {
    key: 'ajax',
    value: function (method, url, data, headers) {
      var _this = this;
      var promise = new _ParsePromise2.default();

      (function () {
        if (this.XHR == null) {
          throw new Error('Cannot make a request: No definition of XMLHttpRequest was found.');
        }
        var handled = false;
        var XHR = this.XHR;
        var xhr = new XHR();

        xhr.onreadystatechange = function () {
          if (xhr.readyState !== 4 || handled) {
            return;
          }
          handled = true;

          if (xhr.status >= 200 && xhr.status < 300) {
            var response;
            try {
              response = JSON.parse(xhr.responseText);
            } catch (e) {
              promise.reject(e.toString());
            }
            if (response) {
              promise.resolve(response, xhr.status, xhr);
            }
          } else {
            promise.reject(xhr);
          }
        };

        headers = headers || {};
        headers['X-Auth-Token'] = _this.datonisToken;
        if (typeof headers['Content-Type'] !== 'string') {
          headers['Content-Type'] = 'application/json'; // Avoid pre-flight
        }

        xhr.open(method, url, true);
        for (var h in headers) {
          xhr.setRequestHeader(h, headers[h]);
        }
        xhr.send(data);
      })();

      return promise;
    }
  }, {
    key: 'getDatonisUrl',
    value: function () {
      var datonisUrl = this._CoreManager.get('DATONIS_URL');
      if (datonisUrl[datonisUrl.length - 1] !== '/') {
        datonisUrl += '/';
      }
      return datonisUrl;
    }
  }]);
  return DatonisFacade;
}();

exports.default = DatonisFacade;