'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LiveQueryFacade = undefined;

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _EventEmitter2 = require('./EventEmitter');

var _EventEmitter3 = _interopRequireDefault(_EventEmitter2);

var _LiveQueryClient = require('./LiveQueryClient');

var _LiveQueryClient2 = _interopRequireDefault(_LiveQueryClient);

var _CoreManager = require('./CoreManager');

var _ParsePromise = require('./ParsePromise');

var _ParsePromise2 = _interopRequireDefault(_ParsePromise);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

function getSessionToken(coreManager) {
  var controller = coreManager.getUserController();
  return controller.currentUserAsync().then(function (currentUser) {
    return currentUser ? currentUser.getSessionToken() : undefined;
  });
}

function getLiveQueryClient(coreManager) {
  return coreManager.getLiveQueryController().getDefaultLiveQueryClient();
}

/**
 *
 * We expose three events to help you monitor the status of the WebSocket connection:
 *
 * <p>Open - When we establish the WebSocket connection to the LiveQuery server, you'll get this event.
 * 
 * <pre>
 * Parse.LiveQuery.on('open', () => {
 * 
 * });</pre></p>
 *
 * <p>Close - When we lose the WebSocket connection to the LiveQuery server, you'll get this event.
 * 
 * <pre>
 * Parse.LiveQuery.on('close', () => {
 * 
 * });</pre></p>
 *
 * <p>Error - When some network error or LiveQuery server error happens, you'll get this event.
 * 
 * <pre>
 * Parse.LiveQuery.on('error', (error) => {
 * 
 * });</pre></p>
 * 
 * @class Parse.LiveQuery
 * @static
 * 
 */

var LiveQueryFacade = exports.LiveQueryFacade = function (_EventEmitter) {
  (0, _inherits3.default)(LiveQueryFacade, _EventEmitter);

  function LiveQueryFacade(coreManager) {
    (0, _classCallCheck3.default)(this, LiveQueryFacade);

    var _this = (0, _possibleConstructorReturn3.default)(this, (LiveQueryFacade.__proto__ || (0, _getPrototypeOf2.default)(LiveQueryFacade)).call(this));

    _this._CoreManager = coreManager;
    _this.on('error', function () {});
    _this._CoreManager.setLiveQueryController(new DefaultLiveQueryController(_this._CoreManager, _this));
    return _this;
  }

  (0, _createClass3.default)(LiveQueryFacade, [{
    key: 'open',
    value: function () {
      var LiveQueryController = this._CoreManager.getLiveQueryController();
      LiveQueryController.open();
    }
  }, {
    key: 'close',
    value: function () {
      var LiveQueryController = this._CoreManager.getLiveQueryController();
      LiveQueryController.close();
    }
  }]);
  return LiveQueryFacade;
}(_EventEmitter3.default);

var DefaultLiveQueryController = function () {
  function DefaultLiveQueryController(coreManager, liveQuery) {
    (0, _classCallCheck3.default)(this, DefaultLiveQueryController);

    this._CoreManager = coreManager;
    this._LiveQuery = liveQuery;
  }

  (0, _createClass3.default)(DefaultLiveQueryController, [{
    key: 'setDefaultLiveQueryClient',
    value: function (liveQueryClient) {
      this.defaultLiveQueryClient = liveQueryClient;
    }
  }, {
    key: 'getDefaultLiveQueryClient',
    value: function () {
      var _this2 = this;

      if (this.defaultLiveQueryClient) {
        return _ParsePromise2.default.as(this.defaultLiveQueryClient);
      }

      return getSessionToken(this._CoreManager).then(function (sessionToken) {
        var liveQueryServerURL = _this2._CoreManager.get('LIVEQUERY_SERVER_URL');

        if (liveQueryServerURL && liveQueryServerURL.indexOf('ws') !== 0) {
          throw new Error('You need to set a proper Parse LiveQuery server url before using LiveQueryClient');
        }

        // If we can not find Parse.liveQueryServerURL, we try to extract it from Parse.serverURL
        if (!liveQueryServerURL) {
          var tempServerURL = _this2._CoreManager.get('SERVER_URL');
          var protocol = 'ws://';
          // If Parse is being served over SSL/HTTPS, ensure LiveQuery Server uses 'wss://' prefix
          if (tempServerURL.indexOf('https') === 0) {
            protocol = 'wss://';
          }
          var host = tempServerURL.replace(/^https?:\/\//, '');
          liveQueryServerURL = protocol + host;
          _this2._CoreManager.set('LIVEQUERY_SERVER_URL', liveQueryServerURL);
        }

        var applicationId = _this2._CoreManager.get('APPLICATION_ID');
        var javascriptKey = _this2._CoreManager.get('JAVASCRIPT_KEY');
        var masterKey = _this2._CoreManager.get('MASTER_KEY');
        // Get currentUser sessionToken if possible
        _this2.defaultLiveQueryClient = new _LiveQueryClient2.default({
          applicationId: applicationId,
          serverURL: liveQueryServerURL,
          javascriptKey: javascriptKey,
          masterKey: masterKey,
          sessionToken: sessionToken,
          coreManager: _this2._CoreManager
        });
        // Register a default onError callback to make sure we do not crash on error
        // Cannot create these events on a nested way because of EventEmiiter from React Native
        _this2.defaultLiveQueryClient.on('error', function (error) {
          _this2._LiveQuery.emit('error', error);
        });
        _this2.defaultLiveQueryClient.on('open', function () {
          _this2._LiveQuery.emit('open');
        });
        _this2.defaultLiveQueryClient.on('close', function () {
          _this2._LiveQuery.emit('close');
        });

        return _this2.defaultLiveQueryClient;
      });
    }
  }, {
    key: 'open',
    value: function () {
      var _this3 = this;

      getLiveQueryClient(this._CoreManager).then(function (liveQueryClient) {
        _this3.resolve(liveQueryClient.open());
      });
    }
  }, {
    key: 'close',
    value: function () {
      var _this4 = this;

      getLiveQueryClient(this._CoreManager).then(function (liveQueryClient) {
        _this4.resolve(liveQueryClient.close());
      });
    }
  }, {
    key: 'subscribe',
    value: function (query) {
      var _this5 = this;

      var subscriptionWrap = new _EventEmitter3.default();

      getLiveQueryClient(this._CoreManager).then(function (liveQueryClient) {
        if (liveQueryClient.shouldOpen()) {
          liveQueryClient.open();
        }
        var promiseSessionToken = getSessionToken(_this5._CoreManager);
        // new event emitter
        return promiseSessionToken.then(function (sessionToken) {

          var subscription = liveQueryClient.subscribe(query, sessionToken);
          // enter, leave create, etc

          subscriptionWrap.id = subscription.id;
          subscriptionWrap.query = subscription.query;
          subscriptionWrap.sessionToken = subscription.sessionToken;
          subscriptionWrap.unsubscribe = subscription.unsubscribe;
          // Cannot create these events on a nested way because of EventEmiiter from React Native
          subscription.on('open', function () {
            subscriptionWrap.emit('open');
          });
          subscription.on('create', function (object) {
            subscriptionWrap.emit('create', object);
          });
          subscription.on('update', function (object) {
            subscriptionWrap.emit('update', object);
          });
          subscription.on('enter', function (object) {
            subscriptionWrap.emit('enter', object);
          });
          subscription.on('leave', function (object) {
            subscriptionWrap.emit('leave', object);
          });
          subscription.on('delete', function (object) {
            subscriptionWrap.emit('delete', object);
          });
          subscription.on('close', function (object) {
            subscriptionWrap.emit('close', object);
          });
          subscription.on('error', function (object) {
            subscriptionWrap.emit('error', object);
          });

          _this5.resolve();
        });
      });
      return subscriptionWrap;
    }
  }, {
    key: 'unsubscribe',
    value: function (subscription) {
      var _this6 = this;

      getLiveQueryClient(this._CoreManager).then(function (liveQueryClient) {
        _this6.resolve(liveQueryClient.unsubscribe(subscription));
      });
    }
  }, {
    key: '_clearCachedDefaultClient',
    value: function () {
      this.defaultLiveQueryClient = null;
    }
  }]);
  return DefaultLiveQueryController;
}();

var LiveQuery = new LiveQueryFacade(_CoreManager.CoreManager);

exports.default = LiveQuery;