'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RESTController = exports.RESTControllerFacade = undefined;

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _CoreManager = require('./CoreManager');

var _ParseError = require('./ParseError');

var _ParseError2 = _interopRequireDefault(_ParseError);

var _ParsePromise = require('./ParsePromise');

var _ParsePromise2 = _interopRequireDefault(_ParsePromise);

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj };
}

function ajaxIE9(method, url, data) {
  var promise = new _ParsePromise2.default();
  var xdr = new XDomainRequest();
  xdr.onload = function () {
    var response;
    try {
      response = JSON.parse(xdr.responseText);
    } catch (e) {
      promise.reject(e);
    }
    if (response) {
      promise.resolve(response);
    }
  };
  xdr.onerror = xdr.ontimeout = function () {
    // Let's fake a real error message.
    var fakeResponse = {
      responseText: (0, _stringify2.default)({
        code: _ParseError2.default.X_DOMAIN_REQUEST,
        error: 'IE\'s XDomainRequest does not supply error info.'
      })
    };
    promise.reject(fakeResponse);
  };
  xdr.onprogress = function () {};
  xdr.open(method, url);
  xdr.send(data);
  return promise;
} /**
   * Copyright (c) 2015-present, Parse, LLC.
   * All rights reserved.
   *
   * This source code is licensed under the BSD-style license found in the
   * LICENSE file in the root directory of this source tree. An additional grant
   * of patent rights can be found in the PATENTS file in the same directory.
   *
   * 
   */

var RESTControllerFacade = exports.RESTControllerFacade = function () {
  function RESTControllerFacade(coreManager) {
    (0, _classCallCheck3.default)(this, RESTControllerFacade);

    this._CoreManager = coreManager;
    this._CoreManager.setRESTController(this);

    this.XHR = null;
    if (typeof XMLHttpRequest !== 'undefined') {
      this.XHR = XMLHttpRequest;
    }

    this.XHR = require('xmlhttprequest').XMLHttpRequest;


    this.useXDomainRequest = false;
    if (typeof XDomainRequest !== 'undefined' && !('withCredentials' in new XMLHttpRequest())) {
      this.useXDomainRequest = true;
    }
  }

  (0, _createClass3.default)(RESTControllerFacade, [{
    key: 'ajax',
    value: function (method, url, data, headers) {
      var _this = this;

      if (this.useXDomainRequest) {
        return ajaxIE9(method, url, data, headers);
      }

      var promise = new _ParsePromise2.default();
      var attempts = 0;

      (function dispatch() {
        if (_this.XHR == null) {
          throw new Error('Cannot make a request: No definition of XMLHttpRequest was found.');
        }
        var handled = false;
        var xhrClazz = _this.XHR;
        var xhr = new xhrClazz();

        xhr.onreadystatechange = function () {
          if (xhr.readyState !== 4 || handled) {
            return;
          }
          handled = true;

          if (xhr.status >= 200 && xhr.status < 300) {
            var response;
            try {
              response = JSON.parse(xhr.responseText);
            } catch (e) {
              promise.reject(e.toString());
            }
            if (response) {
              promise.resolve(response, xhr.status, xhr);
            }
          } else if (xhr.status >= 500 || xhr.status === 0) {
            // retry on 5XX or node-xmlhttprequest error
            if (++attempts < _this._CoreManager.get('REQUEST_ATTEMPT_LIMIT')) {
              // Exponentially-growing random delay
              var delay = Math.round(Math.random() * 125 * Math.pow(2, attempts));
              setTimeout(dispatch, delay);
            } else if (xhr.status === 0) {
              promise.reject('Unable to connect to the Parse API');
            } else {
              // After the retry limit is reached, fail
              promise.reject(xhr);
            }
          } else {
            promise.reject(xhr);
          }
        };

        headers = headers || {};
        if (typeof headers['Content-Type'] !== 'string') {
          headers['Content-Type'] = 'text/plain'; // Avoid pre-flight
        }
        if (_this._CoreManager.get('IS_NODE')) {
          headers['User-Agent'] = 'Parse/' + _this._CoreManager.get('VERSION') + ' (NodeJS ' + process.versions.node + ')';
        }

        xhr.open(method, url, true);
        for (var h in headers) {
          xhr.setRequestHeader(h, headers[h]);
        }
        xhr.send(data);
      })();

      return promise;
    }
  }, {
    key: 'request',
    value: function (method, path, data, options) {
      var _this2 = this;

      options = options || {};
      var url = this._CoreManager.get('SERVER_URL');
      if (url[url.length - 1] !== '/') {
        url += '/';
      }
      url += path;

      var payload = {};
      if (data && (typeof data === 'undefined' ? 'undefined' : (0, _typeof3.default)(data)) === 'object') {
        for (var k in data) {
          payload[k] = data[k];
        }
      }

      if (method !== 'POST') {
        payload._method = method;
        method = 'POST';
      }

      payload._ApplicationId = this._CoreManager.get('APPLICATION_ID');
      var jsKey = this._CoreManager.get('JAVASCRIPT_KEY');
      if (jsKey) {
        payload._JavaScriptKey = jsKey;
      }
      payload._ClientVersion = this._CoreManager.get('VERSION');

      var useMasterKey = options.useMasterKey;
      if (typeof useMasterKey === 'undefined') {
        useMasterKey = this._CoreManager.get('USE_MASTER_KEY');
      }
      if (useMasterKey) {
        if (this._CoreManager.get('MASTER_KEY')) {
          delete payload._JavaScriptKey;
          payload._MasterKey = this._CoreManager.get('MASTER_KEY');
        } else {
          throw new Error('Cannot use the Master Key, it has not been provided.');
        }
      }

      if (this._CoreManager.get('FORCE_REVOCABLE_SESSION')) {
        payload._RevocableSession = '1';
      }

      var installationId = options.installationId;
      var installationIdPromise;
      if (installationId && typeof installationId === 'string') {
        installationIdPromise = _ParsePromise2.default.as(installationId);
      } else {
        var installationController = this._CoreManager.getInstallationController();
        installationIdPromise = installationController.currentInstallationId();
      }

      return installationIdPromise.then(function (iid) {
        payload._InstallationId = iid;
        var userController = _this2._CoreManager.getUserController();
        if (options && typeof options.sessionToken === 'string') {
          return _ParsePromise2.default.as(options.sessionToken);
        } else if (userController) {
          return userController.currentUserAsync().then(function (user) {
            if (user) {
              return _ParsePromise2.default.as(user.getSessionToken());
            }
            return _ParsePromise2.default.as(null);
          });
        }
        return _ParsePromise2.default.as(null);
      }).then(function (token) {
        if (token) {
          payload._SessionToken = token;
        }

        var payloadString = (0, _stringify2.default)(payload);

        return _this2.ajax(method, url, payloadString);
      }).then(null, function (response) {
        // Transform the error into an instance of ParseError by trying to parse
        // the error string as JSON
        var error;
        if (response && response.responseText) {
          try {
            var errorJSON = JSON.parse(response.responseText);
            error = new _ParseError2.default(errorJSON.code, errorJSON.error);
          } catch (e) {
            // If we fail to parse the error text, that's okay.
            error = new _ParseError2.default(_ParseError2.default.INVALID_JSON, 'Received an error with invalid JSON from Parse: ' + response.responseText);
          }
        } else {
          error = new _ParseError2.default(_ParseError2.default.CONNECTION_FAILED, 'XMLHttpRequest failed: ' + (0, _stringify2.default)(response));
        }

        return _ParsePromise2.default.error(error);
      });
    }
  }, {
    key: '_setXHR',
    value: function (xhr) {
      this.XHR = xhr;
    }
  }]);
  return RESTControllerFacade;
}();

var RESTController = exports.RESTController = new RESTControllerFacade(_CoreManager.CoreManager);

exports.default = RESTController;