/**
 * Copyright (c) 2015-present, Parse, LLC.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

import { CoreManagerInstance, CoreManager } from './CoreManager';
import ParsePromise from './ParsePromise';

var iidCache = null;

function hexOctet() {
  return Math.floor((1 + Math.random()) * 0x10000).toString(16).substring(1);
}

function generateId() {
  return hexOctet() + hexOctet() + '-' + hexOctet() + '-' + hexOctet() + '-' + hexOctet() + '-' + hexOctet() + hexOctet() + hexOctet();
}

export class InstallationControllerInstance {
  constructor(coreManager) {
    this._CoreManager = coreManager;
  }

  currentInstallationId() {
    if (typeof iidCache === 'string') {
      return ParsePromise.as(iidCache);
    }
    var path = this._CoreManager.getStorageFacade().generatePath('installationId');
    return this._CoreManager.getStorageFacade().getItemAsync(path).then(iid => {
      if (!iid) {
        iid = generateId();
        return this._CoreManager.getStorageFacade().setItemAsync(path, iid).then(() => {
          iidCache = iid;
          return iid;
        });
      }
      iidCache = iid;
      return iid;
    });
  }

  _clearCache() {
    iidCache = null;
  }

  _setInstallationIdCache(iid) {
    iidCache = iid;
  }
}

const InstallationController = new InstallationControllerInstance(CoreManager);
export default InstallationController;